from PyQt5.QtCore import QUrl
from PyQt5.QtGui import QIcon, QDesktopServices
from PyQt5.QtWidgets import QAction, QLabel, QMessageBox, QInputDialog
from qgis.core import QgsMessageLog, QgsVectorLayer, QgsProject, QgsGeometry,\
    QgsFeature, QgsComposition
from qgis.gui import QgsMapToolIdentifyFeature, QgsMapToolEmitPoint
from .db import get_connection
# Initialize Qt resources from file resources.py
from .resources import *
from .paggf_layers import LoadedLayers
from .paggf_utils import PAGGFUtils
import os.path
import tempfile
import base64
from . import paggf_version
from .ui.CID_load_attr_from_shp import LoadAttrFromFile
from PyQt5.QtXml import QDomDocument
import inspect
from .composer.ComposerValidation import ComposerValidation


class PAGGFToolbar(object):
    toolbars = dict()
    plugin = None

    @classmethod
    def create(cls, name, ui_name):
        """

        Keyword arguments:
        name -- Id of the toolbar
        ui_name -- Name displayed in the UI
        """
        PAGGFToolbar.toolbars[name] = cls.plugin.iface.addToolBar(ui_name)
        PAGGFToolbar.toolbars[name].setObjectName(ui_name)

    @classmethod
    def create_from_list(cls, l):
        for name, ui_name in l:
            cls.create(name, ui_name)

    @classmethod
    def delete(cls, n):
        del cls.toolbars[n]


class PAGGFAction(object):
    plugin = None  # Acces au plugin
    active = None  # si une action reste active a la fin du callback
    # la désactiver au lancement de la nouvelle action
    actions = dict()  # Lancement des actions

    def __init__(
        self, name, menu_title, icon_path=None, callback=None,
        before_callback=None, after_callback=None,
        config=None, active_after_callback=False, stop_action=None
    ):
        self.name = name
        self.menu_title = menu_title
        self.icon_path = icon_path
        self.callback = callback
        self.before_callback = before_callback  # to execute before callback
        self.after_callback = after_callback  # to execute after callback
        self.config = config
        self.active_after_callback = active_after_callback
        self.stop_action = stop_action

        self.action = None  # au debut action pas activee
        PAGGFAction.actions[self.name] = self

    def toolbar_name(self):
        if self.config and 'tb_name' in self.config:
            return self.config['tb_name']
        return None

    def activate(self):
        icon = QIcon(self.icon_path)
        self.action = QAction(
            icon, self.menu_title, PAGGFAction.plugin.iface.mainWindow())
        self.action.triggered.connect(self.exec_callback)

        tb_name = self.toolbar_name()
        if tb_name and tb_name in PAGGFToolbar.toolbars:
            PAGGFToolbar.toolbars[tb_name].addAction(self.action)

        self.action.setCheckable(True)

    def exec_callback(self, *args, **kwargs):
        prev_active_action = PAGGFAction.active

        if PAGGFAction.active:
            PAGGFAction.active.action.setChecked(False)

            if PAGGFAction.active.stop_action:
                PAGGFAction.active.stop_action()
                PAGGFAction.active = None

        if prev_active_action == self:  # On desactive l'action car second
            # click sur l'icone
            pass
        else:
            self.exec_before_callback()

            sig = inspect.signature(self.callback)

            if len(sig.parameters) == 0:
                self.callback()
            else:
                self.callback(*args, **kwargs)

            self.exec_after_callback()

            if self.active_after_callback:
                PAGGFAction.active = self

    def exec_before_callback(self):
        if self.before_callback:
            self.before_callback()

    def exec_after_callback(self):
        if self.after_callback:
            self.after_callback()


class ClickDlgAction(PAGGFAction):
    """
    Action that is trigger when clicking on a icon, then display a dlg,
    then finish.
    """
    def __init__(self, *args, **kwargs):
        super(ClickDlgAction, self).__init__(*args, **kwargs)

        self.after_callback = self.after_callback_fct

    def after_callback_fct(self):
        self.action.setChecked(False)


class ActionForFeatureSelected(PAGGFAction):
    # Action dont l'icon est active que quand feature est selectionne)
    def __init__(self, *args, **kwargs):
        super(ActionForFeatureSelected, self).__init__(*args, **kwargs)

    def activate(self):
        super(ActionForFeatureSelected, self).activate()

        self.enable_action_when_feature_selected()

    def enable_action_when_feature_selected(self):
        """
        Enable an action when one and only one feature is selected in a given
        layer
        """
        paggf_schema_layer = self.config['pg_schema_name']
        paggf_table_layer = self.config['pg_table_name']

        layer = self.plugin.layers.get(paggf_schema_layer, paggf_table_layer)

        def get_set_slot_checkable(layer, action):
            def ret():
                if layer.selectedFeatureCount() == 1:
                    self.action.setEnabled(True)
                else:
                    self.action.setEnabled(False)
            return ret

        get_set_slot_checkable(layer, self)()
        layer.selectionChanged.connect(
            get_set_slot_checkable(layer, self))

def dirty_layer(layer):
    """
    Modifiy a layer so that it is considered dirty by QGIS
    and the current transaction won't be empty
    """
    try:
        feature = next(layer.getFeatures())
        # sets id to id just to dirty buffer
        layer.editBuffer().changeAttributeValue(
            feature.id(),
            feature.fieldNameIndex('id'),
            feature.attribute('id'))
    except StopIteration:
        pass

def sql_quote_nullable(v):
    if v is None:
        return "null"
    elif isinstance(v, str):
        return "'" + v.replace("'", "''") + "'"
    return v

def execute_sql_on_layer(layer, sql):
    """
    Execute a SQL query on the database of the given layer.
    The query will be inserted in the transaction object of the
    layer
    """
    # switch to edit mode if not already the case
    layer.startEditing()
    transaction = layer.dataProvider().transaction()
    transaction.executeSql(sql)


class PAGGFCore(object):
    """Encodes the CID inventory"""

    def __init__(self, iface, version, menu_title):
        """
        Constructor

        Keyword arguments:
        iface -- the QgsInterface
        version -- version of the plugin (BM, CID or EF see paggf_version.py)
        menu_title -- the tile of the menu
        """
        self.iface = iface
        self.version = version
        self.menu_title = menu_title

        self.mapCanvas = iface.mapCanvas()

        self.load_parcel_dlg = None
        self.load_attributes_dlg = None

        self.layers = LoadedLayers()

        self.maptool = None

        self.actions = dict()  # Dictionary where are stored the actions
        self.actions_configuration = dict()  # configuration of the
        # PAGGF actions
        self.actions_def = dict()

        self.active_action_name = None  # The name of the action which is
        # active at the  moment
        self.stop_action_function = dict()  # Dictionary of function to
        # execute at the end of an action

        self.plugin_dir = os.path.dirname(__file__)

        self.check_composer_window = None  # composer to check before export
        self.check_composition = None  # composition for checking before export

        self.utils = PAGGFUtils(self.layers)

        PAGGFAction.plugin = self
        PAGGFToolbar.plugin = self

    def enable_paggf_action(self, action_id, configuration):
        """
        Enable a paggf action. To be called during __init__.

        Keyword arguments:
        action_id -- The id of the action
        configuration -- its configuration
        """
        self.actions_configuration[action_id] = configuration

    def create_paggf_menu(self):
        """
        Create the PAGGF menu (the tree of the menu is defined in the
        varialbe self.paggf_menu_content)
        """
        self.paggf_menu = self.iface.mainWindow().menuBar().addMenu(
            self.menu_title)
        self.paggf_menu.setObjectName('mPAGGF')

        def def_recursive_create_menu(parent, children):
            """
            Ajout recursif d'un menu
            parent  -> deja un menu
            la suite (à ajouter) soit
                - une liste (sous-menu)
                - une action (fin du la branche du menu)
                - un label (action pas encore ecrite)
            """
            for child in children:
                if isinstance(child, PAGGFAction):  # fin de l'arble ->
                    # ajoute l'action
                    parent.addAction(child.action)
                elif type(child) == tuple:  # ajout sous menu
                    c_label, c_children = child
                    c_menu = parent.addMenu(c_label)
                    def_recursive_create_menu(c_menu, c_children)
                elif type(child) == str:  # Ajout d'une action non ecrite
                    action = parent.addAction(child)
                    action.setEnabled(False)
                else:  # Error
                    print(child)
                    print(type(child))

        def_recursive_create_menu(self.paggf_menu, self.paggf_menu_content())

    def initGui(self):
        """Create the menu entries and toolbar icons inside the QGIS GUI."""

        PAGGFToolbar.create_from_list(self.toolbar_list_def())

        for action_name in self.actions_configuration:
            config = self.actions_configuration[action_name]

            if action_name == 'load_parcel_from_file':
                callback = self.get_load_parcel_callback(config)
                ClickDlgAction(
                    action_name,
                    config['menu_title'],
                    ':/plugins/PAGGF/img/icon_add_parcel.svg',
                    callback, config=config)

            elif action_name == 'load_parcel_manually':
                ClickDlgAction(
                    action_name,
                    config['menu_title'],
                    ':/plugins/PAGGF/img/icon_capture_parcel.svg',
                    self.get_load_parcel_callback(config), config=config)

            elif action_name == 'add_vertex':
                PAGGFAction(
                    action_name,
                    'Ajouter un sommet',
                    ':/plugins/PAGGF/img/icon_add_vertex.png',
                    self.select_point_then_trigger_action_factory(
                        'gis', 'segments_idc_parcels_to_valid',
                        self.add_vertex_at),
                    active_after_callback=True,
                    stop_action=self.pan_trigger, config=config)

            elif action_name == 'move_vertex':
                PAGGFAction(
                    action_name,
                    'Déplacer les sommets',
                    ':/plugins/PAGGF/img/icon_move_vertex.png',
                    self.move_vertex,
                    active_after_callback=True,
                    stop_action=self.pan_trigger, config=config)

            elif action_name == 'delete_vertex':
                PAGGFAction(
                    action_name,
                    'Supprimer les sommets',
                    ':/plugins/PAGGF/img/icon_delete_vertex.png',
                    self.select_point_then_trigger_action_factory(
                        'gis', 'idc_vertices_to_valid', self.delete_vertex_at),
                    active_after_callback=True,
                    stop_action=self.pan_trigger, config=config)

            elif action_name == 'add_ring':
                PAGGFAction(
                    action_name,
                    'Ajouter un anneau',
                    ':/plugins/PAGGF/img/icon_add_ring.svg',
                    self.add_ring,
                    active_after_callback=True,
                    stop_action=self.pan_trigger, config=config)

            elif action_name == 'reshape':
                PAGGFAction(
                    action_name,
                    'Remodeler les entités',
                    ':/plugins/PAGGF/img/icon_reshape.svg',
                    self.reshape,
                    active_after_callback=True,
                    stop_action=self.pan_trigger, config=config)

            elif action_name == 'edit_data':
                PAGGFAction(
                    action_name,
                    config['menu_title'],
                    ':/plugins/PAGGF/img/icon_form_view.svg',
                    self.edit_data,
                    active_after_callback=True,
                    stop_action=self.pan_trigger, config=config)

            elif action_name == 'print_pv':
                PAGGFAction(
                    action_name,
                    config['menu_title'],
                    ':/plugins/PAGGF/img/icon_print_pv.png',
                    self.select_point_then_trigger_action_factory(
                        config['pg_schema_name'], config['pg_table_name'],
                        self.print_pv_for_parcel_at),
                    active_after_callback=True,
                    stop_action=self.pan_trigger, config=config)

            elif action_name == 'archive_parcel':
                a = PAGGFAction(
                    action_name,
                    'Archiver une parcelle',
                    ':/plugins/PAGGF/img/icon_archive.svg',
                    self.select_point_then_trigger_action_factory(
                        config['pg_schema_name'], config['pg_table_name'],
                        self.archive_parcel_at),
                    active_after_callback=True,
                    stop_action=self.pan_trigger, config=config)

            elif action_name == 'copy_parcel':
                a = PAGGFAction(
                    action_name,
                    'Copier une parcelle archivée',
                    ':/plugins/PAGGF/img/icon_archive_copy.svg',
                    self.select_point_then_trigger_action_factory(
                        config['pg_schema_name'], config['pg_table_name'],
                        self.copy_parcel_at),
                    active_after_callback=True,
                    stop_action=self.pan_trigger, config=config)

            elif action_name == 'pv_upload_signatures':
                ActionForFeatureSelected(
                    action_name,
                    config['menu_title'],
                    ':/plugins/PAGGF/img/icon_sign.svg',
                    self.pv_upload_signatures,
                    config=config)

            elif action_name == 'pv_w_signatures_print':
                ActionForFeatureSelected(
                    action_name,
                    config['menu_title'],
                    ':/plugins/PAGGF/img/icon_print_pv2.png',
                    self.pv_w_signatures_print,
                    config=config)

            elif action_name == 'load_attributes_from_file':
                ActionForFeatureSelected(
                    action_name,
                    config['menu_title'],
                    ':/plugins/PAGGF/img/icon_upload.png',
                    self.loadattributes_callback,
                    config=config)

            elif action_name == 'composer_validation':
                ActionForFeatureSelected(
                    action_name,
                    config['menu_title'],
                    ':/plugins/PAGGF/img/icon_composer_validation.svg',
                    self.composer_validation,
                    config=config)

            elif action_name == 'composer_map_munip_inventory':
                ClickDlgAction(
                    action_name,
                    config['menu_title'],
                    ':/plugins/PAGGF/img/icon_composer_validation.svg',
                    self.composer_map_munip_inventory,
                    config=config)

            else:
                print('action {} not found'.format(create_action))

            PAGGFAction.actions[action_name].activate()

        self.create_paggf_menu()

        dlg = self.welcome_dlg_class()
        dlg.exec_()

    def unload(self):
        """Removes the plugin menu item and icon from QGIS GUI."""
        for paggf_action in PAGGFAction.actions.values():
            self.iface.removeToolBarIcon(paggf_action.action)
        # remove the toolbar

        toolbar_names = [n for n in PAGGFToolbar.toolbars]
        for n in toolbar_names:
            PAGGFToolbar.delete(n)

        # remove the menu
        self.paggf_menu.clear()
        self.paggf_menu.deleteLater()

    def get_load_parcel_callback(self, config):
        def ret():
            dlg_class = config['dlg_class']
            widget_class = config['load_widget_class']

            if (not self.load_parcel_dlg) or \
                    (not self.load_parcel_dlg.isVisible()):
                self.load_parcel_dlg = dlg_class(
                    data_loader_class=widget_class,
                    mapCanvas=self.mapCanvas,
                    version=self.version)
                self.load_parcel_dlg.show()
                self.load_parcel_dlg.exec_()
        return ret

    def select_point_then_trigger_action_factory(
        self, db_schema_name, db_table_name, action_to_trigger):

        layer = self.layers.get(db_schema_name, db_table_name)

        def on_feature_clicked(point_clicked):
            pt = QgsGeometry.fromPoint(point_clicked)
            pt_buffer = pt.buffer(
                (self.mapCanvas.mapUnitsPerPixel() * 2), 0)
            pt_rect = pt_buffer.boundingBox()

            features = layer.getFeatures(pt_rect)

            feature = QgsFeature()
            if features.nextFeature(feature):
                # make sure the iterator is closed to avoid locks
                features.close()
                # call callback
                action_to_trigger(layer, point_clicked, feature)

        def select_point_then_trigger_action():
            vertex_layer = self.layers.get(db_schema_name, db_table_name)
            self.iface.setActiveLayer(vertex_layer)
            self.maptool = QgsMapToolEmitPoint(self.mapCanvas)
            self.maptool.canvasClicked.connect(on_feature_clicked)
            self.mapCanvas.setMapTool(self.maptool)

        return select_point_then_trigger_action

    def add_vertex_at(self, layer, point_clicked, edge):
        add_vertex_layer = self.layers.get(
            'gis', 'idc_vertices_to_valid')
        add_vertex_layer.startEditing()

        pt_wkt_txt = "'srid=32735;point({} {})'::geometry"\
            .format(point_clicked.x(), point_clicked.y())
        sql = 'SELECT gis.add_idc_vertex_to_parcel({},{},{},{})'\
            .format(
                edge['id_parcel'], edge['id_vertex_before'],
                edge['id_vertex_after'], pt_wkt_txt)
        execute_sql_on_layer(add_vertex_layer, sql)
        dirty_layer(add_vertex_layer)
        add_vertex_layer.triggerRepaint()

    def move_vertex(self):
        vertex_layer = self.layers.get('gis', 'idc_vertices_to_valid')
        self.iface.setActiveLayer(vertex_layer)
        vertex_layer.startEditing()
        self.iface.actionMoveFeature().trigger()

    def delete_vertex_at(self, vertex_layer, point_clicked, feat):
        vertex_layer.deleteFeature(feat.id())
        self.mapCanvas.refresh()

    def add_ring(self, feat):
        """
        Trigger the addRing action on the layer idc_parcels_to_valid. This
        action allow the user to create ring into a polygon
        """
        parcels_layer = self.layers.get('gis', 'idc_parcels_to_valid')
        self.iface.setActiveLayer(parcels_layer)
        parcels_layer.startEditing()
        self.iface.actionAddRing().trigger()

    def reshape(self, feat):
        """
        Trigger the reshape / the cut feature action on the layer
        idc_parcels_to_valid. This action allow the user to cut external part
        of a polygone
        """
        parcels_layer = self.layers.get('gis', 'idc_parcels_to_valid')
        self.iface.setActiveLayer(parcels_layer)
        parcels_layer.startEditing()
        reshapeAction = [
            o for o in self.iface.mainWindow().findChildren(QAction)
            if o.objectName() == 'mActionReshapeFeatures'][0]
        reshapeAction.trigger()

    def edit_data(self):
        config = self.actions_configuration['edit_data']
        parcels_layer = self.layers.get(
            config['pg_schema_name'], config['pg_table_name'])
        self.iface.setActiveLayer(parcels_layer)
        parcels_layer.startEditing()
        self.iface.actionIdentify().trigger()

    def print_pv_for_parcel_at(self, layer, point_clicked, parcel):
        config = self.actions_configuration['print_pv']
        out_path = os.path.join(
            tempfile.gettempdir(),
            "inventory_{}.pdf".format(parcel['id']))

        pv_generator = config['pv_generator_cls'](self.plugin_dir, parcel)
        pv_generator.create_pdf_at(out_path)

        # ask the OS to open the generated PDF
        QDesktopServices.openUrl(QUrl.fromLocalFile(out_path))

    def archive_parcel_at(self, layer, point_clicked, parcel):
        r = QMessageBox.warning(None,
                                "Confirmation d'archivage",
                                "L'archivage d'une parcelle est une opération \
irréversible. Confirmez-vous l'archivage ?",
                                QMessageBox.Yes | QMessageBox.No)
        if r == QMessageBox.No:
            return

        event = QInputDialog.getText(
            None, "Motif d'archivage", "Motif d'archivage")

        execute_sql_on_layer(layer,
            ("update gis.idc_parcels_to_valid set " +
             "validity_period = tstzrange(now(), 'infinity', '[)'), " +
             "validity_event = {} where id = {}").format(sql_quote_nullable(event), parcel['id']))

        self.iface.mapCanvas().refreshAllLayers()

    def copy_parcel_at(self, parcel_layer, point_clicked, parcel):
        execute_sql_on_layer(parcel_layer,
            "select copy_object('land.parcels', {})".format(parcel['id']))
        self.iface.mapCanvas().refreshAllLayers()

    def pv_upload_signatures(self):
        from .ui.CID_merge_pv_and_signatures import CIDPvMergeDlg
        from .ui.file_custom_widget import FileCustomWidget

        config = self.actions_configuration['pv_upload_signatures']
        parcel_layer = self.layers.get(
            config['pg_schema_name'], config['pg_table_name'])

        if parcel_layer.selectedFeatureCount() == 1:
            parcel_layer.startEditing()
            feature = parcel_layer.selectedFeatures()[0]

            self.upload_doc_dlg = CIDPvMergeDlg()
            self.upload_doc_widget = FileCustomWidget(self.upload_doc_dlg)

            file_custom_widget_label = QLabel()
            file_custom_widget_label.setBuddy(self.upload_doc_widget)
            file_custom_widget_label.setText('Enregistrement des signatures')

            self.upload_doc_dlg.formLayout.addRow(
                file_custom_widget_label, self.upload_doc_widget)

            self.upload_doc_widget.setEnabled(True)
            self.upload_doc_widget.set_data_base64(
                feature['doc_idc_report_signatures'])

            def update_file_widget(layer, feature):
                # update custom file
                layer.changeAttributeValue(
                    feature.id(),
                    layer.fields().indexFromName("doc_idc_report_signatures"),
                    self.upload_doc_widget.data_base64())

            self.upload_doc_dlg.buttonBox.accepted.connect(
                lambda lyr=parcel_layer, f=feature: update_file_widget(lyr, f)
            )

            def download_empty_signatures(parcel):
                def ret():
                    generated_pv_filename = os.path.join(
                        tempfile.gettempdir(),
                        "inventory_pv_{}.pdf".format(parcel['id']))

                    pv_generator = config['pv_generator_cls'](
                        self.plugin_dir, parcel)
                    pv_generator.create_pdf_at(generated_pv_filename)

                    empty_signatures_filename = os.path.join(
                        tempfile.gettempdir(),
                        "inventory_{}_signatures.pdf".format(parcel['id']))

                    with open(generated_pv_filename, "rb") as generated_pv_file,\
                            open(empty_signatures_filename, 'wb') as empty_sign_file:
                        from .deps.PyPDF2__v1_26_0.PyPDF2 import PdfFileReader, PdfFileWriter
                        pv_reader = PdfFileReader(generated_pv_file)
                        writer = PdfFileWriter()
                        for page in pv_reader.pages[4:]:
                            writer.addPage(page)
                        writer.write(empty_sign_file)

                    QDesktopServices.openUrl(
                        QUrl.fromLocalFile(empty_signatures_filename))
                return ret

            self.upload_doc_dlg.download_empty_signatures_button.clicked \
                .connect(download_empty_signatures(feature))

            def download_unsigned_pv(parcel):
                def ret():
                    generated_pv_filename = os.path.join(
                        tempfile.gettempdir(),
                        "inventory_pv_non_signe_{}.pdf".format(parcel['id']))

                    pv_generator = config['pv_generator_cls'](
                        self.plugin_dir, parcel)
                    pv_generator.create_pdf_at(generated_pv_filename)

                    QDesktopServices.openUrl(
                        QUrl.fromLocalFile(generated_pv_filename))
                return ret

            self.upload_doc_dlg.download_unsigned_pv.clicked.connect(
                download_unsigned_pv(feature))

            self.upload_doc_dlg.exec_()
        else:
            msg_bar = self.iface.messageBar()
            msg_bar.pushCritical('Erreur', 'veuillez sélectionner un 1 et 1 \
seul élément')

    def pv_w_signatures_print(self):
        config = self.actions_configuration['pv_w_signatures_print']
        parcel_layer = self.layers.get(
            config['pg_schema_name'], config['pg_table_name'])

        if parcel_layer.selectedFeatureCount() == 1:
            from .deps.PyPDF2__v1_26_0.PyPDF2 import PdfFileMerger
            merger = PdfFileMerger()
            parcel = parcel_layer.selectedFeatures()[0]

            if parcel['doc_idc_report_signatures']:
                generated_pv_filename = os.path.join(
                    tempfile.gettempdir(),
                    "inventory_generated_pv_{}.pdf".format(parcel['id']))

                signalture_filename, signature_b64data = \
                    parcel['doc_idc_report_signatures'].split(';')

                signature_filename = os.path.join(
                    tempfile.gettempdir(), signalture_filename)

                pv_w_signatues_filename = os.path.join(
                    tempfile.gettempdir(),
                    "inventory_pv_{}.pdf".format(parcel['id']))

                pv_generator = config['pv_generator_cls'](
                    self.plugin_dir, parcel)
                pv_generator.create_pdf_at(generated_pv_filename)

                with open(generated_pv_filename, "rb") as generated_pv_file,\
                        open(signature_filename, 'wb+') as signature_file,\
                        open(pv_w_signatues_filename, 'wb') as pv_w_signatures_file:
                    merger.append(fileobj=generated_pv_file, pages=(0, 4))
                    signature_file.write(
                        base64.b64decode(signature_b64data))
                    merger.append(signature_file)
                    merger.write(pv_w_signatures_file)

                QDesktopServices.openUrl(
                    QUrl.fromLocalFile(pv_w_signatues_filename))
            else:
                msg_bar = self.iface.messageBar()
                msg_bar.pushCritical(
                    'Erreur', "cette parcelle n'a pas de signatures, vous ne \
pouvez l'imprimer")
        else:
            msg_bar = self.iface.messageBar()
            msg_bar.pushCritical(
                'Erreur', 'veuillez sélectionner un 1 et 1 seul élément')

    def pan_trigger(self):
        self.iface.actionPan().trigger()

    def loadattributes_callback(self):
        config = self.actions_configuration['load_attributes_from_file']
        parcel_layer = self.layers.get(
            config['pg_schema_name'], config['pg_table_name'])

        if parcel_layer.selectedFeatureCount() == 1:
            self.load_attributes_dlg = LoadAttrFromFile(
                parcel_layer.selectedFeatures()[0],
                parcel_layer)
            self.load_attributes_dlg.exec_()
        else:
            msg_bar = self.iface.messageBar()
            msg_bar.pushCritical(
                'Erreur',
                'Veuillez sélectionner une et une seule parcelle')

    def composer_validation(self):
        parcel_layer = self.layers.get('gis', 'idc_parcels_to_valid')

        if parcel_layer.selectedFeatureCount() == 1:
            f = parcel_layer.selectedFeatures()[0]

            template_path = os.path.join(
                self.plugin_dir, 'composer/A4_composer.qpt')

            self.cv = ComposerValidation(
                f, template_path, self.iface, zoom_to_feature=True)
            self.cv.substitution_map['QGIS_PLUGIN_DIR'] = self.plugin_dir
            self.cv.exec_()
        else:
            print('\
Veuillez choisir un et un seul élement dans la couche\
gis.idc_parcels_to_valid')

    def composer_map_munip_inventory(self):
        municipality = self.utils.get_current_municipality()
        province = self.utils.get_current_province()

        template_path = os.path.join(
            self.plugin_dir, 'composer/CID_A0_municipality_inventory.qpt')

        self.cv = ComposerValidation(
            municipality, template_path, self.iface, zoom_to_feature=True)

        self.cv.substitution_map['municipality'] = municipality['name']
        self.cv.substitution_map['province'] = province['name']
        self.cv.substitution_map['QGIS_PLUGIN_DIR'] = self.plugin_dir

        self.cv.exec_()
