import os

from PyQt5 import uic
from PyQt5 import QtWidgets
from PyQt5.QtCore import QUrl, QVariant
from PyQt5.QtGui import QDesktopServices, QIcon, QImage, QPixmap
from PyQt5.QtWidgets import QWidget, QFileDialog

from qgis.gui import QgsEditorConfigWidget, QgsEditorWidgetWrapper, QgsEditorWidgetFactory
from qgis.core import QgsFieldFormatter

import base64
import tempfile

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'file_custom_widget.ui'))

class FileCustomWidget(QtWidgets.QWidget, FORM_CLASS):
    def __init__(self, parent=None):
        super(FileCustomWidget, self).__init__(parent)
        self.setupUi(self)
        self.openButton.setIcon(QIcon(os.path.join(os.path.dirname(__file__),"..","img","mActionFileOpen.svg")))
        self.saveAsButton.setIcon(QIcon(os.path.join(os.path.dirname(__file__),"..","img","mActionFileSaveAs.svg")))
        self.replaceButton.setIcon(QIcon(os.path.join(os.path.dirname(__file__),"..","img","mActionFileNew.svg")))
        self.clearButton.setIcon(QIcon(os.path.join(os.path.dirname(__file__),"..","img","mActionDeleteSelected.svg")))
        self._data = None

        self.openButton.clicked.connect(self.on_open)
        self.saveAsButton.clicked.connect(self.on_save_as)
        self.replaceButton.clicked.connect(self.on_replace)
        self.clearButton.clicked.connect(self.on_clear)

        self._extension_icons = {'pdf' : os.path.join(os.path.dirname(__file__),"..","img","pdf.svg") }

    def set_filename(self, fn):
        if fn is None:
            self.fileNameEdit.setText("<No file>")
            self.iconLbl.hide()
        else:
            self.fileNameEdit.setText(fn)
            if fn[-3:] in self._extension_icons:
                img = QImage(self._extension_icons[fn[-3:]]).scaled(32, 32)
                self.iconLbl.setPixmap(QPixmap.fromImage(img))
                self.iconLbl.show()

    def _data(self):
        return self._data
        
    def _set_data(self, filename, data):
        """
        filename: string
        data: bytes
        """
        self.set_filename(filename)
        self._data = data
        self.openButton.setEnabled(True)
        self.saveAsButton.setEnabled(True)
        self.clearButton.setEnabled(True)

    def data_base64(self):
        if self._data is None:
            return ''
        return "{};{}".format(self.fileNameEdit.text().replace(";",""),
                              base64.b64encode(self._data).decode("ascii"))

    def set_data_base64(self, data):
        if data == '' or (isinstance(data, QVariant) and data.isNull()):
            self.set_filename(None)
            self._data = None
            self.openButton.setEnabled(False)
            self.saveAsButton.setEnabled(False)
            self.clearButton.setEnabled(False)
        else:
            filename, b64data = data.split(';')
            self.set_filename(filename)
            self._data = base64.b64decode(b64data)
            self.openButton.setEnabled(True)
            self.saveAsButton.setEnabled(True)
            self.clearButton.setEnabled(True)

    def setEnabled(self, enabled):
        self.fileNameEdit.setEnabled(enabled)
        self.replaceButton.setEnabled(enabled)
        self.clearButton.setEnabled(enabled)

    def on_open(self):
        if self._data is None:
            return
        dtemp = tempfile.mkdtemp()
        fn = os.path.join(dtemp, self.fileNameEdit.text())
        with open(fn, "wb") as f:
            f.write(self._data)
        QDesktopServices.openUrl(QUrl.fromLocalFile(fn))

    def on_save_as(self):
        if self._data is None:
            return
        fn, filter = QFileDialog.getSaveFileName(None, "Select file to save to")
        if fn == '':
            return
        with open(fn, "wb") as f:
            f.write(self._data)

    def on_replace(self):
        fn, filter = QFileDialog.getOpenFileName(None, "Select file to store")
        if fn == '':
            return
        with open(fn, "rb") as f:
            self._set_data(os.path.basename(fn), f.read())

    def on_clear(self):
        self._data = None
        self.set_filename(None)


class FileCustomWidgetWrapper(QgsEditorWidgetWrapper):
    def __init__(self, vl, fieldIdx, editor, parent):
        super(FileCustomWidgetWrapper, self).__init__(vl, fieldIdx, editor, parent)
        self._file_widget = editor

    def createWidget(self, parent):
        self._file_widget = FileCustomWidget(parent)
        return self._file_widget

    def value(self):
        return self._file_widget.data_base64()

    def setValue(self, v):
        self._file_widget.set_data_base64(v)

    def initWidget(self, editor):
        self._file_widget = editor

    def valid(self):
        return isinstance(self._file_widget, FileCustomWidget)

    def setEnabled(self, b):
        self._file_widget.setEnabled(b)

class FileWidgetConfigDlg(QgsEditorConfigWidget):
    def __init__(self, vl, fieldIdx, parent):
        super(FileWidgetConfigDlg, self).__init__(vl, fieldIdx, parent)

    def config(self):
        return {}

    def setConfig(self, cfg):
        pass


class FileWidgetFactory(QgsEditorWidgetFactory):
    def __init__(self):
        super(FileWidgetFactory, self).__init__("Base64 file")

    def create(self, vl, fieldIdx, editor, parent):
        # QgsVectorLayer* vl, int fieldIdx, QWidget* editor, QWidget* parent
        self.wrapper = FileCustomWidgetWrapper(vl, fieldIdx, editor, parent)
        return self.wrapper

    def configWidget(self, vl, fieldIdx, parent ):
        self.dlg = FileWidgetConfigDlg(vl, fieldIdx, parent)
        return self.dlg

class FileWidgetFormatter(QgsFieldFormatter):
    def id(self):
        return "Base64 file"

    def representValue(self, layer, fieldIdx, config, cache, value):
        return "<...>"
