"""
Functions to works with drag&drop forms generated by QGIS
"""
from PyQt5.QtWidgets import QLabel
from qgis.gui import QgsEditorWidgetWrapper


def my_form_open(dialog, layer, feature):
    print (get_label_and_buddy(dialog))
    print (get_editor_widget_wrapper(dialog))


def search_descendants(obj, find_cond, get_key, get_value):
    """
    Returns a dictionary containing the descendants of an object
    (children, children of children, ...) that respect a given condition

    Keyword arguments:
    obj -- The object
    find_cond -- The condition : the child c has to be returned if
        find_cond(c) is True
    get_key -- A function that returns for a child its key to add in the
        returned dict
    get_value -- A function that returns for a child its value to add in the
        returned dict
    """
    ret_dict = dict()
    for c in obj.children():
        ret_dict.update(search_descendants(
            c, find_cond, get_key, get_value))

    if find_cond(obj):
        ret_dict[get_key(obj)] = get_value(obj)

    return ret_dict


def get_form_labels(form):
    """
    This function return the list of the Qlabel (and then their associated
    Qfield) of a drag&drop QGIS generated form
    """
    def find_cond(el):
        return type(el) == QLabel and el.buddy()

    def get_key(el):
        return el.buddy().objectName()

    def get_value(el):
        return el

    return search_descendants(form, find_cond, get_key, get_value)


def get_editor_widget_wrappers(form):
    """
    This function return the list of QgsEditorWidgetWrapper of the
    drag&drop QGIS generated form
    """
    def find_cond(el):
        return type(el) == QgsEditorWidgetWrapper

    def get_key(el):
        return el.field().name()

    def get_value(el):
        return el

    return search_descendants(form, find_cond, get_key, get_value)


class QgsDragDropForm(object):
    def __init__(self, dlg):
        self.form_labels = get_form_labels(dlg)
        self.editor_widget_wrappers = get_editor_widget_wrappers(dlg)

    def get_field_label(self, field_name):
        """Get the QLabel for the field having field_name as 'name'"""
        return self.form_labels[field_name]

    def get_field_editor_widget(self, field_name):
        """Get the QField for the field having field_name as 'name'"""
        return (self.form_labels[field_name]).buddy()

    def get_field_editor_wrapper(self, field_name):
        """
        Get the QgsEditorWidgetWrapper for the field having field_name
        as 'name'
        """
        return self.editor_widget_wrappers[field_name]

    def register_when_field_display_other(
            self, field_for_cond, values, field_to_display):
        """
        When the field 'field_for_cond' has a value in the array 'values'
        display the 'field_for_cond'
        """
        ew = self.get_field_editor_wrapper(field_for_cond)

        def value_changed_slot(new_val):
            if new_val in values:
                self.get_field_editor_widget(field_to_display).show()
                self.get_field_label(field_to_display).show()
            else:
                self.get_field_editor_widget(field_to_display).hide()
                self.get_field_label(field_to_display).hide()

        ew.valueChanged.connect(value_changed_slot)
