import os

from PyQt5 import uic
from PyQt5.QtCore import QSettings
from PyQt5.QtWidgets import QDialog, QFileDialog, QMessageBox
from qgis.core import QgsMessageLog, QgsVectorLayer, QgsSettings

from PAGGF.db import get_connection

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'CID_load_attr_from_shp.ui'))

fields_map_from_to = [
    # (field_name_in_shp, field_name_in_db)
    # (field_name_in_shp, field_name_in_db, enum_map, other_field)
    ('DATE', 'begin_date'),
    ('DATE', 'end_date'),
    ('U/L', 'usual_name'),
    ('PROP', 'owner', {"etat": 'State',
                       "commune" : "Municipality",
                       "etablissement public" : "PublicInstitution",
                       # other field
                       None: "Other"}, "owner_other"),
    ('GEST', 'manager', {'etat':'State',
                         'personne physique': 'NaturalPerson',
                         "personne morale de droit prive": "LegalPerson",
                         "commune": "Municipality",
                         "etablissement public": "PublicInstitution",
                         None: "Other"}, "manager_other"),
    ('OD', 'ownership_origin', {"propre à l'etat": 'State',
                                "achat": "Buying",
                                "cession": "Recovery",
                                "don et leg": "Donation",
                                "expropriation": "Expropriation",
                                "echange": "Transfer",
                                None: "Other"}, "origin_other"),
    ('USA', 'usage', {"boisement": "Afforestation",
                      "centre de sante": "HealthCenre",
                      "cimetière": "Cemetery",
                      "cimétière": "Cemetery",
                      "ecofo": "NaturalForest",
                      "ecole secondaire": "SecondarySchool",
                      "marais": "Swamp",
                      "marché": "Market",
                      "prairie": "Pasture",
                      "terrain de jeux": "PlayArea",
                      None: "Other"}, "usage_other"),
    ('RJ', 'legal_regime', {'domaine privé' : 'Private', 'domaine public': 'Public'}, None),
    ('DC', 'rights', {'location' : 'Rent',
                      'amphythéose': 'Emphyteusis',
                      'habitation': 'Housing',
                      'servitude': 'Easement',
                      'hypothèque': 'Mortgage',
                      None: 'None'}, None),
    ('EM', 'promotion', {'non exploité': 'None',
                         'partiel': 'Partial',
                         'total': 'Total',
                         None: 'None'}, None),
    ('AF', 'assignment', {'boisement' : 'Afforestation',
                          'prairie': 'Pasture',
                          'marché': 'Marché',
                          'ecole secondaire': 'SecondarySchool',
                          'ecole primaire': 'PrimarySchool',
                          'cimetière': 'Cemetery',
                          'cimétière': 'Cemetery',
                          'terrain de jeux': 'PlayArea',
                          'village batwa': 'BatwaVillage',
                          'centre de sante': 'HealthCentre',
                          'forêt naturelle': 'NaturalForest',
                          'parc national': 'NationalPark',
                          'réserve naturelle': 'NaturalReserve',
                          'marais': 'Swamp',
                          None: 'Other'}, "assignment_other"),
    ('OC', 'conflict_type', {'pas de conflit': 'None',
                             'conflit de limite' : 'Boundary',
                             'conflit de propriété' : 'Ownership',
                             'conflit de limite et propriété': 'BoundaryAndOwnership'}, None),
    ('PC', 'conflicting_parts', {'aucune': 'None',
                                 'etat et particulier': 'StateIndividual',
                                 'etat et personne morale de droit prive': 'StateLegalPerson',
                                 'etat et commune' : 'StateMunicipality',
                                 'etat et etablissement public' : 'StatePublicInstit',
                                 'etablissement public et etablissement public' : 'PublicInstitPublicInstit',
                                 'etablissement public et commune' : 'PublicInstitMunicipality',
                                 None: 'None'}, None),
    ('OBS', 'observations')
]

class LoadAttrFromFile(QDialog, FORM_CLASS):
    def __init__(self, feature_to, layer_to, parent=None):
        # feature_to -> qui va recevoir les données
        # layer_to -> qui va recevoir
        # message_bar -> pour communiquer
        super(LoadAttrFromFile, self).__init__()
        self.setupUi(self)
        self.feature_to = feature_to
        self.layer_to = layer_to

        self.column_from_selector = 'ID'
        # display this colum to the use for making his choice
        # FIXME if there is more than one feature,

        self.browse_button.clicked.connect(self.file_browse)
        self.button_box.accepted.connect(self.load_attributes_slot)

        self._layer_from = None
        self._layer_from_dp = None

    def file_browse(self):
        s = QgsSettings("PAGGF", "PAGGF")
        last_file_path = s.value("lastPAGGFFileLoaded", ".")

        if not os.path.exists(last_file_path):
            last_file_path = os.getcwd()

        file_from_path, _ = QFileDialog.getOpenFileName(
            self, "Fichier contenant les attributs à charger",
            last_file_path, "*.shp *.xls *.xlsx")

        s.setValue("lastPAGGFFileLoaded", file_from_path)

        self._layer_from = QgsVectorLayer(
            file_from_path, 'load_attibutes_layer', 'ogr')
        self._layer_from_dp = self._layer_from.dataProvider()

        # check that the file contains requested fields
        fields = [m[0] for m in fields_map_from_to]
        for field in fields:
            if self._layer_from_dp.fields().indexOf(field) == -1:
                msg = QMessageBox()
                msg.setText(
                    'Le fichier {} ne possède pas de colonne {}'.format(
                        file_from_path,
                        field
                    )
                )
                msg.exec_()
                return

        self.update_file_path_field(file_from_path)
        self.update_column_from_selector()

    def update_column_from_selector(self):
        if self._layer_from_dp.fieldNameIndex(self.column_from_selector) != -1:
            if self._layer_from:
                self.feature_selector.clear()
                self.feature_selector.addItem('')
                for f in self._layer_from.getFeatures():
                    self.feature_selector.addItem(
                        str(f[self.column_from_selector]))
        else:
            msg = QMessageBox()
            msg.setText(
                'Le fichier {} ne possède pas de colonne {}'.format(
                    file_from_path,
                    self.column_from_selector
                )
            )
            msg.exec_()

    def update_file_path_field(self, file_from_path):
        self.file_path_field.setText(file_from_path)

    def load_attributes_slot(self):
        row_selector = self.feature_selector.currentText()

        if row_selector:
            for f in self._layer_from.getFeatures():
                if str(f[self.column_from_selector]) == row_selector:
                    self.load_attributes_from_feature(f)
                    self.load_adjacent_people(f)
                    break
        else:
            msg = QMessageBox()
            msg.setText("Veuillez choisir l'élément à importer")
            msg.exec_()

    def load_attributes_from_feature(self, feature_from):
        self.layer_to.startEditing()

        layer_from_dp = self._layer_from.dataProvider()
        layer_to_dp = self.layer_to.dataProvider()

        changes = []

        for m in fields_map_from_to:
            enum_map = None
            if len(m) == 2:
                # from, to
                fieldname_from, fieldname_to = m
            elif len(m) == 4:
                # from, to, enum_map, other_field
                fieldname_from, fieldname_to, enum_map, fieldname_other = m
            origin_value = feature_from[fieldname_from]
            dest_value = origin_value
            if enum_map is not None:
                if enum_map.get(origin_value.lower()) is not None:
                    dest_value = enum_map[origin_value.lower()]
                else:
                    # no mapping
                    # set to_field to "Other"
                    # and other_field to the value
                    dest_value = enum_map.get(None, None)
                    if fieldname_other is not None:
                        changes.append((fieldname_other, origin_value))
            changes.append((fieldname_to, dest_value))

            for fieldname_to, new_value in changes:
                self.layer_to.changeAttributeValue(
                    self.feature_to.id(),
                    layer_to_dp.fieldNameIndex(fieldname_to),
                    new_value)

    def load_adjacent_people(self, feature):
        conn = get_connection()
        cur = conn.cursor()
        people = [x.strip().split(' ') for x in feature['LM'].split(',')]
        people = "row(" + "),row(".join(["'{}','{}'".format(p[0], ' '.join(p[1:]) if len(p)>1 else '') for p in people]) + ")"
        cur.execute("\
        SELECT gis.inventory_set_adjacent_people({}, array[{}]::gis._adjacent_people_param[])".format(
            self.feature_to["id_inventory"], people))
        conn.commit()
